/* ex: set ro ft=c:
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from './src/pmc/complex.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

/* HEADERIZER HFILE: none */
/* HEADERIZER STOP */

#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_fixedintegerarray.h"
#include "pmc_complex.h"
#include "pmc_default.h"
#include "complex.str"
#include "pmc_continuation.h"
#line 1 "./src/pmc/complex.pmc"
/*
Copyright (C) 2004-2009, Parrot Foundation.
$Id: complex.pmc 38964 2009-05-20 04:49:37Z petdance $

=head1 NAME

src/pmc/complex.pmc - Complex Numbers PMC Class

=head1 DESCRIPTION

C<Complex> provides a representation of complex numbers. It handles
string parsing/generating and basic mathematical operations.

=head2 Functions

=over 4

=cut

Equations used are sometimes listed.  At times, multiple equations are given,
but those starting with => are the ones used

*/

/*

=item C<static void
complex_parse_string(PARROT_INTERP, FLOATVAL *re, FLOATVAL *im, STRING *value)>

Parses the string in C<value> to produce a complex number, represented
by the real (C<*re>) and imaginary (C<*im>) parts. Raises an exception
if it cannot understand the string.  The string should be of the form
C<a+bi> with optional spaces around C<+> and before C<i>. You can also
use C<j> instead of C<i>.

=cut

*/

static void
complex_parse_string(PARROT_INTERP, FLOATVAL *re, FLOATVAL *im, STRING *value) {
    char   * const str        = Parrot_str_to_cstring(interp, value);
    char   *t                 = str;
    char   *first_num_offset  = str;
    char   *second_num_offset = NULL;
    STRING *S;

    INTVAL  i                 = 0;
    INTVAL  first_num_minus   = 0;
    INTVAL  second_num_minus  = 0;
    UINTVAL first_num_length, second_num_length;

    /* walk the string and identify the real and imaginary parts */

    if (*t == '-') {
        /* first number is negative */
        t++;
        first_num_minus = 1;

        /* allow for an optional space */
        if (*t == ' ')
            t++;
        first_num_offset = t;
    }

    /* skip digits */
    while (*t >= '0' && *t <= '9')
        t++;

    if (*t == '.') {
        /* this number has a decimal point */
        t++;

        /* skip digits */
        while (*t >= '0' && *t <= '9')
            t++;
    }

    /* save the length of the real part */
    first_num_length = t - first_num_offset;

    /* end of string; we only have a real part */
    if (*t == 0) {
        second_num_length = 0;
    }
    else if ((*t == 'i' || *t == 'j') && *(t+1) == 0) {
        /* there is only an imaginary part, so the first number was
            actually the imaginary part */
        second_num_length = first_num_length;
        first_num_length  = 0;
        second_num_offset = first_num_offset;
        second_num_minus  = first_num_minus;
        first_num_minus   = 0;

        /* this is useful if there is no number for
            the imaginary part, like in "-i" */
        i = 1;
    }
    else {
        /* skip an optional space */
        if (*t == ' ')
            t++;

        /* expect "+" or "-" and the imaginary part */
        if (*t == '+' || *t == '-') {
            /* save the sign */
            second_num_minus = (*t == '-');
            t++;

            /* skip another optional space */
            if (*t == ' ')
                t++;

            /* save the beginning of the imaginary part */
            second_num_offset = t;

            /* skip digits */
            while (*t >= '0' && *t <= '9')
                t++;

            if (*t == '.') {
                /* this number has a decimal point */
                t++;

                /* skip digits */
                while (*t >= '0' && *t <= '9')
                    t++;
            }

            /* save the length of the imaginary part */
            second_num_length = t - second_num_offset;

            /* allow for one more optional space */
            if (*t == ' ')
                t++;

            /* verify that the string ends properly */
            if ((*t != 'i' && *t != 'j') || (*(t+1) != 0)) {
                /* imaginary part does not end in 'i' or 'j' */
                Parrot_ex_throw_from_c_args(interp, NULL,
                    EXCEPTION_INVALID_STRING_REPRESENTATION,
                    "Complex: malformed string");
            }

            /* this is useful if there is no number for the
                imaginary part, like in "2+i" */
            i = 1;

            /* all is OK, save the number */
        }
        else {
            /* "+" or "-" not found: error */
            Parrot_str_free_cstring(str);

            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_STRING_REPRESENTATION,
                "Complex: malformed string");
        }
    }

    /* now we have the offsets and the lengths we turn them into float values */

    if (first_num_length) {
        /* there is a real part, interpret it */
        S   = Parrot_str_new(interp, first_num_offset, first_num_length);
        *re = Parrot_str_to_num(interp, S);
    }
    else {
        /* consider the real part 0.0 */
        *re = 0.0;
    }

    if (second_num_length) {
        /* there is an imaginary part, interpret it */
        S   = Parrot_str_new(interp, second_num_offset, second_num_length);
        *im = Parrot_str_to_num(interp, S);
    }
    else {
        /* consider the imaginary part 0.0 */
        if (i) /* the string was something like "1+i" */
            *im = 1.0;
        else
            *im = 0.0;
    }

    if (first_num_minus)
        *re = -*re;

    if (second_num_minus)
        *im = -*im;

    Parrot_str_free_cstring(str);
}

static void
int_check_divide_zero(PARROT_INTERP, INTVAL value) {
    if (value == 0)
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}

static void
float_check_divide_zero(PARROT_INTERP, FLOATVAL value) {
    if (FLOAT_IS_ZERO(value))
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}

static void
complex_check_divide_zero(PARROT_INTERP, PMC *value) {
    /* Throw an exception if we are dividing by zero. Check both the real part
     * and the imaginary part.*/
    if (FLOAT_IS_ZERO(VTABLE_get_number_keyed_int(interp, value, 0))
            && FLOAT_IS_ZERO(VTABLE_get_number_keyed_int(interp, value, 1)))
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}
#line 240 "./src/pmc/complex.c"
static  PMC  * Parrot_Complex_absolute(PARROT_INTERP, PMC *pmc, PMC *dest);
static  PMC * Parrot_Complex_add(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_add_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_clone(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_destroy(PARROT_INTERP, PMC *pmc);
static  PMC * Parrot_Complex_divide(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  INTVAL  Parrot_Complex_get_bool(PARROT_INTERP, PMC *pmc);
static  INTVAL  Parrot_Complex_get_integer(PARROT_INTERP, PMC *pmc);
static  INTVAL  Parrot_Complex_get_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  INTVAL  Parrot_Complex_get_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  FLOATVAL  Parrot_Complex_get_number(PARROT_INTERP, PMC *pmc);
static  FLOATVAL  Parrot_Complex_get_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  FLOATVAL  Parrot_Complex_get_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key);
static  FLOATVAL  Parrot_Complex_get_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  PMC  * Parrot_Complex_get_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  PMC  * Parrot_Complex_get_pmc_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key);
static  PMC  * Parrot_Complex_get_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  STRING  * Parrot_Complex_get_string(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_i_absolute(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_i_add(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_add_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void Parrot_Complex_i_divide(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void Parrot_Complex_i_multiply(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_i_neg(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_i_subtract(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_init(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_init_pmc(PARROT_INTERP, PMC *pmc, PMC *initializer);
static  PMC  * Parrot_Complex_instantiate(PARROT_INTERP, PMC *pmc, PMC *sig);
static  opcode_t  * Parrot_Complex_invoke(PARROT_INTERP, PMC *pmc, void *next);
static  INTVAL Parrot_Complex_is_equal(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC * Parrot_Complex_multiply(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_neg(PARROT_INTERP, PMC *pmc, PMC *dest);
static  PMC * Parrot_Complex_pow(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_set_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key, INTVAL value);
static  void  Parrot_Complex_set_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, INTVAL value);
static  void  Parrot_Complex_set_integer_native(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_set_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key, FLOATVAL value);
static  void  Parrot_Complex_set_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key, FLOATVAL v);
static  void  Parrot_Complex_set_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, FLOATVAL value);
static  void  Parrot_Complex_set_number_native(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_set_pmc(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_set_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key, PMC *value);
static  void  Parrot_Complex_set_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, PMC *value);
static  void  Parrot_Complex_set_string_native(PARROT_INTERP, PMC *pmc, STRING *value);
static  PMC * Parrot_Complex_subtract(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_multi_add_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_add_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_add_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_add_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_subtract_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_subtract_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_subtract_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_subtract_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_multiply_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_multiply_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_multiply_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_multiply_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_divide_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_divide_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_divide_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_divide_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  INTVAL  Parrot_Complex_multi_is_equal_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  INTVAL  Parrot_Complex_multi_is_equal_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  void Parrot_Complex_nci_ln(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_exp(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sin(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cos(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_tan(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cot(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sec(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_csc(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asin(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acos(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_atan(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acot(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acsc(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asec(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sinh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cosh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_tanh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_coth(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_csch(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sech(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asinh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acosh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_atanh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acoth(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acsch(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asech(PARROT_INTERP, PMC *pmc);
static  PMC  * Parrot_Complex_multi_pow_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_pow_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void Parrot_Complex_nci_sqrt(PARROT_INTERP, PMC *pmc);
void    Parrot_Complex_class_init(PARROT_INTERP, int, int);
PARROT_EXPORT VTABLE* Parrot_Complex_update_vtable(ARGMOD(VTABLE*));
PARROT_EXPORT VTABLE* Parrot_Complex_get_vtable(PARROT_INTERP);
void    Parrot_Complex_ro_class_init(PARROT_INTERP, int, int);
PARROT_EXPORT VTABLE* Parrot_Complex_ro_update_vtable(ARGMOD(VTABLE*));
PARROT_EXPORT VTABLE* Parrot_Complex_ro_get_vtable(PARROT_INTERP);
static  PMC  *
Parrot_Complex_absolute(PARROT_INTERP, PMC *pmc, PMC *dest)
{
#line 1231 "./src/pmc/complex.pmc"

    FLOATVAL re, im, d;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    d = sqrt(re*re + im*im);

    dest = pmc_new(interp,
        Parrot_get_ctx_HLL_type(interp, enum_class_Float));

    VTABLE_set_number_native(interp, dest, d);
    return dest;
#line 368 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_add(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_add_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_add_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "add", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 738 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re + value);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 408 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_add_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 734 "./src/pmc/complex.pmc"

    return VTABLE_add_float(interp, pmc, (FLOATVAL)value, dest);
#line 416 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_clone(PARROT_INTERP, PMC *pmc)
{
#line 388 "./src/pmc/complex.pmc"

    PMC * const dest = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 432 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_destroy(PARROT_INTERP, PMC *pmc)
{
#line 383 "./src/pmc/complex.pmc"

    mem_sys_free(PMC_data(pmc));
    PMC_data(pmc) = NULL;
#line 441 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_divide(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_divide_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_divide_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "divide", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 1068 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    float_check_divide_zero(interp, value);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / value);
    SETATTR_Complex_im(interp, dest, im / value);

    return dest;
#line 483 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 1054 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    int_check_divide_zero(interp, value);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / value);
    SETATTR_Complex_im(interp, dest, im / value);

    return dest;
#line 501 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_bool(PARROT_INTERP, PMC *pmc)
{
#line 441 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return !(FLOAT_IS_ZERO(re) && FLOAT_IS_ZERO(im));
#line 512 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer(PARROT_INTERP, PMC *pmc)
{
#line 422 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, pmc);
    return (INTVAL)f;
#line 521 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 468 "./src/pmc/complex.pmc"

    STRING * const s = VTABLE_get_string(interp, key);
    return VTABLE_get_integer_keyed_str(interp, pmc, s);
#line 530 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 473 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number_keyed_str(interp, pmc, key);
    return (INTVAL)f;
#line 539 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number(PARROT_INTERP, PMC *pmc)
{
#line 427 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return sqrt(re * re + im * im);
#line 550 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 478 "./src/pmc/complex.pmc"

    STRING * const s = VTABLE_get_string(interp, key);
    return VTABLE_get_number_keyed_str(interp, pmc, s);
#line 559 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key)
{
#line 549 "./src/pmc/complex.pmc"

    FLOATVAL f;
    switch (key) {
        case 0:
            GETATTR_Complex_re(interp, pmc, f);
            break;
        case 1:
            GETATTR_Complex_im(interp, pmc, f);
            break;
        default:
            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_OPERATION, "Complex: key must be 0 or 1");
    }
    return f;
#line 579 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 483 "./src/pmc/complex.pmc"

    FLOATVAL value;
    if (Parrot_str_equal(interp, key, CONST_STRING(interp, "real"))) {
        GETATTR_Complex_re(interp, pmc, value);
    }
    else if (Parrot_str_equal(interp, key, CONST_STRING(interp, "imag"))) {
        GETATTR_Complex_im(interp, pmc, value);
    }
    else
        Parrot_ex_throw_from_c_args(interp, NULL,
            EXCEPTION_INVALID_OPERATION, "Complex: key is neither 'real' or 'imag'");
    return value;
#line 597 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 497 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, key, CONST_STRING(interp, "Integer"))) {
        const INTVAL i = VTABLE_get_integer(interp, key);
        return VTABLE_get_pmc_keyed_int(interp, pmc, i);
    }
    else {
        STRING * const s = VTABLE_get_string(interp, key);
        return VTABLE_get_pmc_keyed_str(interp, pmc, s);
    }
#line 612 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key)
{
#line 525 "./src/pmc/complex.pmc"

    PMC * const    ret = pmc_new(interp, enum_class_Float);
    const FLOATVAL val = VTABLE_get_number_keyed_int(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;
#line 623 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 508 "./src/pmc/complex.pmc"

    PMC * const    ret = pmc_new(interp, enum_class_Float);
    const FLOATVAL val = VTABLE_get_number_keyed_str(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;
#line 634 "./src/pmc/complex.c"
}
static  STRING  *
Parrot_Complex_get_string(PARROT_INTERP, PMC *pmc)
{
#line 434 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return Parrot_sprintf_c(interp, "%vg%+vgi", re, im);
#line 645 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_absolute(PARROT_INTERP, PMC *pmc)
{
#line 1244 "./src/pmc/complex.pmc"

    FLOATVAL re, im, d;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    d = sqrt(re*re + im*im);
    pmc_reuse(interp, pmc, enum_class_Float, 0);
    VTABLE_set_number_native(interp, pmc, d);
#line 658 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_add(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_add_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_add_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_add", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 774 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re + value);
#line 693 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_add_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 770 "./src/pmc/complex.pmc"

    VTABLE_i_add_float(interp, pmc, (FLOATVAL)value);
#line 701 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_divide(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_divide_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_divide_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_divide", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 1133 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    float_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / value);
    SETATTR_Complex_im(interp, pmc, im / value);
#line 740 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 1122 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    int_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / value);
    SETATTR_Complex_im(interp, pmc, im / value);
#line 755 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_multiply(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_multiply_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_multiply_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_multiply", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 969 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * value);
    SETATTR_Complex_im(interp, pmc, im * value);
#line 792 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 960 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * value);
    SETATTR_Complex_im(interp, pmc, im * value);
#line 805 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_neg(PARROT_INTERP, PMC *pmc)
{
#line 1169 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, -re);
    SETATTR_Complex_im(interp, pmc, -im);
#line 817 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_subtract(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_subtract_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_subtract_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_subtract", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 863 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re - value);
#line 852 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 859 "./src/pmc/complex.pmc"

    VTABLE_i_subtract_float(interp, pmc, (FLOATVAL) value);
#line 860 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_init(PARROT_INTERP, PMC *pmc)
{
#line 342 "./src/pmc/complex.pmc"

    /* XXX should check if mem_sys_allocate failed */
    PMC_data(pmc) = mem_allocate_typed(Parrot_Complex_attributes);
    SETATTR_Complex_re(interp, pmc, 0.0);
    SETATTR_Complex_im(interp, pmc, 0.0);

    PObj_active_destroy_SET(pmc);
#line 873 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_init_pmc(PARROT_INTERP, PMC *pmc, PMC *initializer)
{
#line 351 "./src/pmc/complex.pmc"

    const INTVAL arg_type = VTABLE_type(interp, initializer);
    VTABLE_init(interp, pmc);
    switch (arg_type) {
        case enum_class_String:
            VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, initializer));
            break;
        case enum_class_FixedFloatArray:
        case enum_class_ResizableFloatArray:
        case enum_class_FixedIntegerArray:
        case enum_class_ResizableIntegerArray:
            if (VTABLE_get_integer(interp, initializer) == 2) {
                FLOATVAL re = VTABLE_get_number_keyed_int(interp, initializer, 0);
                FLOATVAL im = VTABLE_get_number_keyed_int(interp, initializer, 1);
                SETATTR_Complex_re(interp, pmc, re);
                SETATTR_Complex_im(interp, pmc, im);
                break;
            }
            /* else let it fall to default */
        default:
            if (VTABLE_isa(interp, initializer, CONST_STRING(interp, "String"))) {
                STRING * s = VTABLE_get_string(interp, initializer);
                VTABLE_set_string_native(interp, pmc, s);
            }
            else {
                Parrot_ex_throw_from_c_args(interp, NULL,
                    EXCEPTION_INVALID_OPERATION,
                    "Invalid Complex initializer");
            }
    }
#line 909 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_instantiate(PARROT_INTERP, PMC *pmc, PMC *sig)
{
#line 235 "./src/pmc/complex.pmc"

    return PMCNULL;

    /* TODO -- really build this thing */
#if 0
    PMC *res = pmc_new(interp, enum_class_Complex);
    FLOATVAL re    = 0.0;
    FLOATVAL im    = 0.0;
    int      argcI = REG_INT(interp, 1);
    int      argcS = REG_INT(interp, 2);
    int      argcP = REG_INT(interp, 3);
    int      argcN = REG_INT(interp, 4);

    /*
     * we can only allow 0..2 arguments of one kind. For
     * mixed (e.g. N, P) args the order of arguments isn't fixed
     */
    if (argcP) {
        re = VTABLE_get_number(interp, REG_PMC(interp, 5));

        if (argcP == 2)
            im = VTABLE_get_number(interp, REG_PMC(interp, 6));
        /* TODO throw exception if argument mismatch */
    }
    else if (argcN) {
        re = REG_NUM(interp, 5);

        if (argcN == 2)
            im = REG_NUM(interp, 6);
    }
    else if (argcI) {
        re = REG_INT(interp, 5);

        if (argcI == 2)
            im = REG_INT(interp, 6);
    }
    else if (argcS == 1) {
        complex_parse_string(interp, &RE(res), &IM(res), REG_STR(interp, 5));
        return res;
    }

    RE(res) = re;
    IM(res) = im;
    return res;
#endif
#line 960 "./src/pmc/complex.c"
}
static  opcode_t  *
Parrot_Complex_invoke(PARROT_INTERP, PMC *pmc, void *next)
{
#line 292 "./src/pmc/complex.pmc"

    const int   argcP = REG_INT(interp, 3);
    PMC * const res   = pmc_new(interp, enum_class_Complex);

    if (argcP == 1) {
        PMC * const arg = REG_PMC(interp, 5);

        if (arg->vtable->base_type == enum_class_String)
            VTABLE_set_string_native(interp, res, VTABLE_get_string(interp, arg));
        else
            SETATTR_Complex_re(interp, res, VTABLE_get_number(interp, arg));
    }
    else if (argcP == 2) {
        SETATTR_Complex_re(interp, res, VTABLE_get_number(interp, REG_PMC(interp, 5)));
        SETATTR_Complex_im(interp, res, VTABLE_get_number(interp, REG_PMC(interp, 6)));
    }

    REG_PMC(interp, 5) = res;
    return (opcode_t *)next;
#line 985 "./src/pmc/complex.c"
}
static  INTVAL
Parrot_Complex_is_equal(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_is_equal_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_is_equal_DEFAULT(interp, pmc, value);
            else {
                INTVAL retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "is_equal", "PP->I", pmc, value, &retval);
                return retval;
            }
            break;

    }
}
static  PMC *
Parrot_Complex_multiply(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_multiply_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "multiply", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 928 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re * value);
    SETATTR_Complex_im(interp, dest, im * value);

    return dest;
#line 1049 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 924 "./src/pmc/complex.pmc"

    return VTABLE_multiply_float(interp, pmc, (FLOATVAL) value, dest);
#line 1057 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_neg(PARROT_INTERP, PMC *pmc, PMC *dest)
{
#line 1156 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, -re);
    SETATTR_Complex_im(interp, dest, -im);

    return dest;
#line 1074 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_pow(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_pow_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_pow_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "pow", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  void 
Parrot_Complex_set_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key, INTVAL value)
{
#line 651 "./src/pmc/complex.pmc"

    VTABLE_set_number_keyed(interp, pmc, key, (FLOATVAL)value);
#line 1106 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, INTVAL value)
{
#line 655 "./src/pmc/complex.pmc"

    VTABLE_set_number_keyed_str(interp, pmc, key, (FLOATVAL)value);
#line 1114 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_native(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 621 "./src/pmc/complex.pmc"

    VTABLE_set_number_native(interp, pmc, (FLOATVAL)value);
#line 1122 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key, FLOATVAL value)
{
#line 659 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, key, CONST_STRING(interp, "Integer"))) {
        const INTVAL i = VTABLE_get_integer(interp, key);
        VTABLE_set_number_keyed_int(interp, pmc, i, value);
    }
    else {
        STRING *s = VTABLE_get_string(interp, key);
        VTABLE_set_number_keyed_str(interp, pmc, s, value);
    }
#line 1137 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key, FLOATVAL v)
{
#line 565 "./src/pmc/complex.pmc"

    switch (key) {
        case 0:
            SETATTR_Complex_re(interp, pmc, v);
            break;
        case 1:
            SETATTR_Complex_im(interp, pmc, v);
            break;
        default:
            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_OPERATION, "Complex: key must be 0 or 1");
    }
#line 1155 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, FLOATVAL value)
{
#line 670 "./src/pmc/complex.pmc"

    if (Parrot_str_equal(interp, key, CONST_STRING(interp, "real"))) {
        SETATTR_Complex_re(interp, pmc, value);
    }
    else if (Parrot_str_equal(interp, key, CONST_STRING(interp, "imag"))) {
        SETATTR_Complex_im(interp, pmc, value);
    }
    else
        Parrot_ex_throw_from_c_args(interp, NULL,
            EXCEPTION_INVALID_OPERATION, "Complex: key is neither 'real' or 'imag'");
#line 1171 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_native(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 625 "./src/pmc/complex.pmc"

    SETATTR_Complex_re(interp, pmc, value);
    SETATTR_Complex_im(interp, pmc, 0.0);
#line 1180 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 609 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, value, CONST_STRING(interp, "Complex"))) {
        FLOATVAL re, im;
        GETATTR_Complex_re(interp, value, re);
        GETATTR_Complex_im(interp, value, im);
        SETATTR_Complex_re(interp, pmc, re);
        SETATTR_Complex_im(interp, pmc, im);
    }
    else
        VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, value));
#line 1196 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key, PMC *value)
{
#line 682 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, value);
    VTABLE_set_number_keyed(interp, pmc, key, f);
#line 1205 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, PMC *value)
{
#line 687 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, value);
    VTABLE_set_number_keyed_str(interp, pmc, key, f);
#line 1214 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_string_native(PARROT_INTERP, PMC *pmc, STRING *value)
{
#line 602 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    complex_parse_string(interp, &re, &im, value);
    SETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_im(interp, pmc, im);
#line 1225 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_subtract(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_subtract_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_subtract_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "subtract", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 827 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re - value);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1265 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 823 "./src/pmc/complex.pmc"

    return VTABLE_subtract_float(interp, pmc, (FLOATVAL)value, dest);
#line 1273 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_add_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 706 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, dest, self_re + val_re);
    SETATTR_Complex_im(interp, dest, self_im + val_im);

    return dest;
#line 1293 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_add_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 722 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re + VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1309 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_add_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 750 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, pmc, self_re + val_re);
    SETATTR_Complex_im(interp, pmc, self_im + val_im);
#line 1326 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_add_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 763 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re + VTABLE_get_number(interp, value));
#line 1337 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_subtract_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 795 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, dest, self_re - val_re);
    SETATTR_Complex_im(interp, dest, self_im - val_im);

    return dest;
#line 1357 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_subtract_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 811 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re - VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1373 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_subtract_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 839 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, pmc, self_re - val_re);
    SETATTR_Complex_im(interp, pmc, self_im - val_im);
#line 1390 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_subtract_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 852 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re - VTABLE_get_number(interp, value));
#line 1401 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_multiply_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 897 "./src/pmc/complex.pmc"

    FLOATVAL a, b, c, d;

    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, a);
    GETATTR_Complex_im(interp, pmc, b);
    GETATTR_Complex_re(interp, value, c);
    GETATTR_Complex_im(interp, value, d);
    SETATTR_Complex_re(interp, dest, a * c - b * d);
    SETATTR_Complex_im(interp, dest, a * d + b * c);

    return dest;
#line 1420 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_multiply_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 912 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re * VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im * VTABLE_get_number(interp, value));

    return dest;
#line 1436 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_multiply_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 940 "./src/pmc/complex.pmc"

    FLOATVAL a, b, c, d;

    GETATTR_Complex_re(interp, pmc, a);
    GETATTR_Complex_im(interp, pmc, b);
    GETATTR_Complex_re(interp, value, c);
    GETATTR_Complex_im(interp, value, d);
    SETATTR_Complex_re(interp, pmc, a * c - b * d);
    SETATTR_Complex_im(interp, pmc, a * d + b * c);
#line 1451 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_multiply_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 951 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, pmc, im * VTABLE_get_number(interp, value));
#line 1464 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_divide_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 1008 "./src/pmc/complex.pmc"

    FLOATVAL mod, re, im;
    FLOATVAL self_re, self_im, val_re, val_im;

    complex_check_divide_zero(interp, value);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    /* a little speed optimisation: cache an intermediate number;
        I'm not sure the compiler does this */

    if (self_im == 0.0 && val_im == 0.0) {
        re = self_re / val_re;
        im = 0.0;
    }
    else {
        mod = (val_re * val_re + val_im * val_im);
        re  = (self_re * val_re + self_im * val_im) / mod;
        im  = (self_im * val_re - self_re * val_im) / mod;
    }

    SETATTR_Complex_re(interp, dest, re);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1499 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_divide_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 1039 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    const FLOATVAL d = VTABLE_get_number(interp, value);
    float_check_divide_zero(interp, d);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / d);
    SETATTR_Complex_im(interp, dest, im / d);

    return dest;
#line 1518 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_divide_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1082 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    FLOATVAL self_re, self_im, val_re, val_im;

    complex_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    if (self_im == 0.0 && val_im == 0.0) {
        re = self_re / val_re;
        im = 0.0;
    }
    else {
        /* a little speed optimisation: cache an intermediate number;
           I'm not sure the compiler does this */
        const FLOATVAL mod = (val_re * val_re + val_im * val_im);
        re  = (self_re * val_re + self_im * val_im) / mod;
        im  = (self_im * val_re - self_re * val_im) / mod;
    }

    SETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_im(interp, pmc, im);

#line 1550 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_divide_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1110 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    const FLOATVAL d = VTABLE_get_number(interp, value);
    float_check_divide_zero(interp, d);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / d);
    SETATTR_Complex_im(interp, pmc, im / d);
#line 1566 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_multi_is_equal_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1187 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);
    return (INTVAL)(self_re == val_re && self_im == val_im);
#line 1579 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_multi_is_equal_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1196 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (im != 0.0)
        return (INTVAL)0;

    return (re == VTABLE_get_number(interp, value));
#line 1594 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_ln(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 1632 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1649 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1274 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    /* This is necessary for atan2 to behave */
    if (im == -0.0)
        im = 0.0;

    result_re = log(sqrt(re*re + im*im));
    if (re == 0.0 && im == 0.0) /* atan2(0, 0) not portable */
        result_im = 0.0;
    else
        result_im = atan2(im, re);

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 1680 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto ln_returns;
    /*END RETURN PMC *d */
    }
#line 1293 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
ln_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 1725 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_exp(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 1763 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1780 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1308 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, f;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    f  = exp(re);

    SETATTR_Complex_re(interp, d, f * cos(im));

    /* If only sin(pi) worked. */
    if (im == 4.0 * atan(1.0)) {
        SETATTR_Complex_im(interp, d, 0.0);
    }
    else {
        SETATTR_Complex_im(interp, d, f * sin(im));
    }

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 1810 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto exp_returns;
    /*END RETURN PMC *d */
    }
#line 1326 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
exp_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 1855 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sin(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 1893 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1910 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1362 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (FLOAT_IS_ZERO(im)) {
        result_re = sin(re);
        result_im = 0.0;
    }
    else if (FLOAT_IS_ZERO(re)) {
        result_re = 0.0;
        result_im = sinh(im);
    }
    else {
        result_re = sin(re) * cosh(im);

        if (im == -0.0)
            result_im = 0.0;
        else
            result_im = cos(re) * sinh(im);

    }

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 1949 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sin_returns;
    /*END RETURN PMC *d */
    }
#line 1389 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sin_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 1994 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cos(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2032 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2049 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1392 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (FLOAT_IS_ZERO(re)) {
        result_re = cosh(im);
        result_im = 0.0;
    }
    else if (FLOAT_IS_ZERO(im)) {
        result_re = cos(re);
        result_im = 0.0;
    }
    else {
        result_re = cos(re) * cosh(im);
        result_im = -1.0 * sin(re) * sinh(im);
    }

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2083 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto cos_returns;
    /*END RETURN PMC *d */
    }
#line 1414 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
cos_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2128 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_tan(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2166 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2183 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1417 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2195 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &d);
#line 1421 "./src/pmc/complex.pmc"

    #line 2199 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &e);
#line 1422 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2210 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto tan_returns;
    /*END RETURN PMC *d */
    }
#line 1426 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
tan_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2255 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cot(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2293 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2310 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1429 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));
    #line 2321 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &d);
#line 1432 "./src/pmc/complex.pmc"

    #line 2325 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &e);
#line 1433 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2336 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto cot_returns;
    /*END RETURN PMC *d */
    }
#line 1437 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
cot_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2381 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sec(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2419 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2436 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1440 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));
    SETATTR_Complex_re(interp, d, 1.0);
    SETATTR_Complex_im(interp, d, 0.0);
    #line 2449 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &e);
#line 1445 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2460 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sec_returns;
    /*END RETURN PMC *d */
    }
#line 1449 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sec_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2505 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_csc(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2543 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2560 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1452 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    SETATTR_Complex_re(interp, d, 1.0);
    SETATTR_Complex_im(interp, d, 0.0);

    #line 2575 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &e);
#line 1459 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2586 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto csc_returns;
    /*END RETURN PMC *d */
    }
#line 1463 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
csc_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2631 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asin(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2669 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2686 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1499 "./src/pmc/complex.pmc"

    FLOATVAL d_re, d_im, e_re, e_im, self_re, self_im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *       e = pmc_new(interp, VTABLE_type(interp, pmc));
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    e      = Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, pmc, e);
    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, e, 1.0 - e_re);
    SETATTR_Complex_im(interp, e, -e_im);

    #line 2707 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, e, CONST_STRING_GEN(interp, "sqrt"), "->P", &d);
#line 1512 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, d, d_re - self_im);
    SETATTR_Complex_im(interp, d, d_im + self_re);

    #line 2716 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &d);
#line 1518 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, e, d_im);
    SETATTR_Complex_im(interp, e, d_re ? -d_re : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 2729 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asin_returns;
    /*END RETURN PMC *e */
    }
#line 1524 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asin_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2774 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acos(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2812 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2829 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1527 "./src/pmc/complex.pmc"

    FLOATVAL d_re, d_im, e_re, e_im, self_re, self_im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *       e = pmc_new(interp, VTABLE_type(interp, pmc));
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    e = Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, pmc, e);
    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, e, 1.0 - e_re);
    SETATTR_Complex_im(interp, e, -e_im);

    #line 2850 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, e, CONST_STRING_GEN(interp, "sqrt"), "->P", &d);
#line 1540 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, d, d_re + self_im);
    SETATTR_Complex_im(interp, d, d_im - self_re);

    #line 2859 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &e);
#line 1546 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, d, e_im + 2.0 * atan(1.0));
    SETATTR_Complex_im(interp, d, e_re ? -e_re : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2872 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acos_returns;
    /*END RETURN PMC *d */
    }
#line 1552 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acos_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2917 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_atan(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 2955 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2972 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1555 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, d_re, d_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, re);
    SETATTR_Complex_im(interp, d, 1 + im);
    SETATTR_Complex_re(interp, e, -re);
    SETATTR_Complex_im(interp, e, 1 - im);

    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 2994 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &d);
#line 1569 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);

    SETATTR_Complex_re(interp, e, (d_im ? d_im : -0.0) / -2.0);
    SETATTR_Complex_im(interp, e, d_re / 2.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3008 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto atan_returns;
    /*END RETURN PMC *e */
    }
#line 1576 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
atan_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3053 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acot(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3091 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3108 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1579 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 3126 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "atan"), "->P", &e);
#line 1589 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3134 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acot_returns;
    /*END RETURN PMC *e */
    }
#line 1590 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acot_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3179 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acsc(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3217 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3234 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1593 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 3252 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "asin"), "->P", &e);
#line 1603 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3260 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acsc_returns;
    /*END RETURN PMC *e */
    }
#line 1604 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acsc_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3305 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asec(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3343 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3360 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1607 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 3378 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "acos"), "->P", &e);
#line 1617 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3386 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asec_returns;
    /*END RETURN PMC *e */
    }
#line 1618 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asec_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3431 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sinh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3469 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3486 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1641 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, sinh(re) * cos(im));
    SETATTR_Complex_im(interp, d, im ? cosh(re) * sin(im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3507 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sinh_returns;
    /*END RETURN PMC *d */
    }
#line 1650 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sinh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3552 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cosh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3590 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3607 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1653 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, cosh(re) * cos(im));
    if (re == 0.0 || im == 0.0) {
        SETATTR_Complex_im(interp, d, 0.0);
    }
    else {
        SETATTR_Complex_im(interp, d, sinh(re) * sin(im));
    }

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3633 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto cosh_returns;
    /*END RETURN PMC *d */
    }
#line 1667 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
cosh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3678 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_tanh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3716 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3733 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1670 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 3745 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sinh"), "->P", &d);
#line 1674 "./src/pmc/complex.pmc"

    #line 3749 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cosh"), "->P", &e);
#line 1675 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3760 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto tanh_returns;
    /*END RETURN PMC *d */
    }
#line 1679 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
tanh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3805 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_coth(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3843 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3860 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1682 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 3872 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "tanh"), "->P", &d);
#line 1686 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3886 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto coth_returns;
    /*END RETURN PMC *d */
    }
#line 1693 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
coth_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3931 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_csch(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 3969 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3986 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1696 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 3998 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sinh"), "->P", &d);
#line 1700 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 4012 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto csch_returns;
    /*END RETURN PMC *d */
    }
#line 1707 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
csch_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4057 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sech(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4095 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4112 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1710 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 4124 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cosh"), "->P", &d);
#line 1714 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 4138 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sech_returns;
    /*END RETURN PMC *d */
    }
#line 1721 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sech_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4183 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asinh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4221 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4238 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1752 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, d,  im);
    SETATTR_Complex_im(interp, d, -re);

    #line 4256 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "asin"), "->P", &d);
#line 1762 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4269 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asinh_returns;
    /*END RETURN PMC *e */
    }
#line 1768 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asinh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4314 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acosh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4352 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4369 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1771 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 4382 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "acos"), "->P", &d);
#line 1776 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4395 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acosh_returns;
    /*END RETURN PMC *e */
    }
#line 1782 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acosh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4440 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_atanh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4478 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4495 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1785 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, d,  im);
    SETATTR_Complex_im(interp, d, -re);

    #line 4513 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "atan"), "->P", &d);
#line 1795 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4526 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto atanh_returns;
    /*END RETURN PMC *e */
    }
#line 1801 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
atanh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4571 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acoth(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4609 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4626 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1804 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 4644 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "atanh"), "->P", &e);
#line 1814 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4652 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acoth_returns;
    /*END RETURN PMC *e */
    }
#line 1815 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acoth_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4697 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acsch(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4735 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4752 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1818 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 4770 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "asinh"), "->P", &e);
#line 1828 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4778 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acsch_returns;
    /*END RETURN PMC *e */
    }
#line 1829 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acsch_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4823 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asech(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 4861 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4878 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1832 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 4896 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "acosh"), "->P", &e);
#line 1842 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4904 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asech_returns;
    /*END RETURN PMC *e */
    }
#line 1843 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asech_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4949 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_pow_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 1865 "./src/pmc/complex.pmc"

    PMC *l = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *log;

    dest = pmc_new(interp, VTABLE_type(interp, pmc));


    Parrot_PCCINVOKE(interp, pmc, CONST_STRING(interp, "ln"),
                "->P", &log);

    l = Parrot_Complex_multi_multiply_Complex_PMC(interp,
                log, value, l);

    Parrot_PCCINVOKE(interp, l, CONST_STRING(interp, "exp"),
                "->P", &dest);

    return dest;
#line 4972 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_pow_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 1884 "./src/pmc/complex.pmc"

    PMC *l = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *log;

    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    Parrot_PCCINVOKE(interp, pmc, CONST_STRING(interp, "ln"),
                 "->P", &log);
    l = Parrot_Complex_multi_multiply_DEFAULT_PMC(interp, log, value, l);
    Parrot_PCCINVOKE(interp, l, CONST_STRING(interp, "exp"),
                 "->P", &dest);

    return dest;
#line 4991 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sqrt(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    Parrot_Context *_caller_ctx   = CONTEXT(interp);
    PMC * const _ret_cont         = new_ret_continuation_pmc(interp, NULL);
    Parrot_Context *_ctx          = Parrot_push_context(interp, _n_regs_used);
    PMC *_ccont                   = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = _caller_ctx->current_cont;
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    _ctx->current_cont            = _ret_cont;
    PMC_cont(_ret_cont)->from_ctx = _ctx;

    _current_args                 = interp->current_args;
    interp->current_args         = NULL;

#line 5029 "./src/pmc/complex.pmc"

#line 440 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        --_ctx->recursion_depth;
        _ctx->caller_ctx      = _caller_ctx->caller_ctx;
        Parrot_free_context(interp, _caller_ctx, 1);
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 5046 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 448 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1899 "./src/pmc/complex.pmc"

    PMC * const result = pmc_new(interp, VTABLE_type(interp, pmc));
    const FLOATVAL absval = pmc->vtable->get_number(interp, pmc);
    FLOATVAL sx, sy, rx, ry;
    GETATTR_Complex_re(interp, pmc, sx);
    GETATTR_Complex_im(interp, pmc, sy);

    rx = sqrt((absval + sx) / 2);
    ry = sqrt((absval - sx) / 2);
    if (sy < 0)
        ry = -ry;
    SETATTR_Complex_re(interp, result, rx);
    SETATTR_Complex_im(interp, result, ry);
    #line 237 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *result */
    /*BEGIN GENERATED ACCESSORS */
#line 5071 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = result;

#line 256 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sqrt_returns;
    /*END RETURN PMC *result */
    }
#line 1912 "./src/pmc/complex.pmc"

#line 457 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 471 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sqrt_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        _caller_ctx->current_results, PARROT_PASS_RESULTS);
#line 481 "/builddir/build/BUILD/parrot-1.4.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 5116 "./src/pmc/complex.c"
}
#include "pmc_default.h"

PARROT_EXPORT
VTABLE *Parrot_Complex_update_vtable(VTABLE *vt) {
    vt->absolute = Parrot_Complex_absolute;
    vt->add = Parrot_Complex_add;
    vt->add_float = Parrot_Complex_add_float;
    vt->add_int = Parrot_Complex_add_int;
    vt->clone = Parrot_Complex_clone;
    vt->destroy = Parrot_Complex_destroy;
    vt->divide = Parrot_Complex_divide;
    vt->divide_float = Parrot_Complex_divide_float;
    vt->divide_int = Parrot_Complex_divide_int;
    vt->get_bool = Parrot_Complex_get_bool;
    vt->get_integer = Parrot_Complex_get_integer;
    vt->get_integer_keyed = Parrot_Complex_get_integer_keyed;
    vt->get_integer_keyed_str = Parrot_Complex_get_integer_keyed_str;
    vt->get_number = Parrot_Complex_get_number;
    vt->get_number_keyed = Parrot_Complex_get_number_keyed;
    vt->get_number_keyed_int = Parrot_Complex_get_number_keyed_int;
    vt->get_number_keyed_str = Parrot_Complex_get_number_keyed_str;
    vt->get_pmc_keyed = Parrot_Complex_get_pmc_keyed;
    vt->get_pmc_keyed_int = Parrot_Complex_get_pmc_keyed_int;
    vt->get_pmc_keyed_str = Parrot_Complex_get_pmc_keyed_str;
    vt->get_string = Parrot_Complex_get_string;
    vt->i_absolute = Parrot_Complex_i_absolute;
    vt->i_add = Parrot_Complex_i_add;
    vt->i_add_float = Parrot_Complex_i_add_float;
    vt->i_add_int = Parrot_Complex_i_add_int;
    vt->i_divide = Parrot_Complex_i_divide;
    vt->i_divide_float = Parrot_Complex_i_divide_float;
    vt->i_divide_int = Parrot_Complex_i_divide_int;
    vt->i_multiply = Parrot_Complex_i_multiply;
    vt->i_multiply_float = Parrot_Complex_i_multiply_float;
    vt->i_multiply_int = Parrot_Complex_i_multiply_int;
    vt->i_neg = Parrot_Complex_i_neg;
    vt->i_subtract = Parrot_Complex_i_subtract;
    vt->i_subtract_float = Parrot_Complex_i_subtract_float;
    vt->i_subtract_int = Parrot_Complex_i_subtract_int;
    vt->init = Parrot_Complex_init;
    vt->init_pmc = Parrot_Complex_init_pmc;
    vt->instantiate = Parrot_Complex_instantiate;
    vt->invoke = Parrot_Complex_invoke;
    vt->is_equal = Parrot_Complex_is_equal;
    vt->multiply = Parrot_Complex_multiply;
    vt->multiply_float = Parrot_Complex_multiply_float;
    vt->multiply_int = Parrot_Complex_multiply_int;
    vt->neg = Parrot_Complex_neg;
    vt->pow = Parrot_Complex_pow;
    vt->set_integer_keyed = Parrot_Complex_set_integer_keyed;
    vt->set_integer_keyed_str = Parrot_Complex_set_integer_keyed_str;
    vt->set_integer_native = Parrot_Complex_set_integer_native;
    vt->set_number_keyed = Parrot_Complex_set_number_keyed;
    vt->set_number_keyed_int = Parrot_Complex_set_number_keyed_int;
    vt->set_number_keyed_str = Parrot_Complex_set_number_keyed_str;
    vt->set_number_native = Parrot_Complex_set_number_native;
    vt->set_pmc = Parrot_Complex_set_pmc;
    vt->set_pmc_keyed = Parrot_Complex_set_pmc_keyed;
    vt->set_pmc_keyed_str = Parrot_Complex_set_pmc_keyed_str;
    vt->set_string_native = Parrot_Complex_set_string_native;
    vt->subtract = Parrot_Complex_subtract;
    vt->subtract_float = Parrot_Complex_subtract_float;
    vt->subtract_int = Parrot_Complex_subtract_int;

    return vt;
}


PARROT_EXPORT
VTABLE *Parrot_Complex_ro_update_vtable(ARGMOD(VTABLE *vt)) {
    vt->absolute = Parrot_Complex_absolute;
    vt->add = Parrot_Complex_add;
    vt->add_float = Parrot_Complex_add_float;
    vt->add_int = Parrot_Complex_add_int;
    vt->clone = Parrot_Complex_clone;
    vt->destroy = Parrot_Complex_destroy;
    vt->divide = Parrot_Complex_divide;
    vt->divide_float = Parrot_Complex_divide_float;
    vt->divide_int = Parrot_Complex_divide_int;
    vt->get_bool = Parrot_Complex_get_bool;
    vt->get_integer = Parrot_Complex_get_integer;
    vt->get_integer_keyed = Parrot_Complex_get_integer_keyed;
    vt->get_integer_keyed_str = Parrot_Complex_get_integer_keyed_str;
    vt->get_number = Parrot_Complex_get_number;
    vt->get_number_keyed = Parrot_Complex_get_number_keyed;
    vt->get_number_keyed_int = Parrot_Complex_get_number_keyed_int;
    vt->get_number_keyed_str = Parrot_Complex_get_number_keyed_str;
    vt->get_pmc_keyed = Parrot_Complex_get_pmc_keyed;
    vt->get_pmc_keyed_int = Parrot_Complex_get_pmc_keyed_int;
    vt->get_pmc_keyed_str = Parrot_Complex_get_pmc_keyed_str;
    vt->get_string = Parrot_Complex_get_string;
    vt->init = Parrot_Complex_init;
    vt->init_pmc = Parrot_Complex_init_pmc;
    vt->instantiate = Parrot_Complex_instantiate;
    vt->invoke = Parrot_Complex_invoke;
    vt->is_equal = Parrot_Complex_is_equal;
    vt->multiply = Parrot_Complex_multiply;
    vt->multiply_float = Parrot_Complex_multiply_float;
    vt->multiply_int = Parrot_Complex_multiply_int;
    vt->neg = Parrot_Complex_neg;
    vt->pow = Parrot_Complex_pow;
    vt->subtract = Parrot_Complex_subtract;
    vt->subtract_float = Parrot_Complex_subtract_float;
    vt->subtract_int = Parrot_Complex_subtract_int;

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
VTABLE* Parrot_Complex_get_vtable(PARROT_INTERP) {
    VTABLE *vt;
    vt = Parrot_default_get_vtable(interp);
    Parrot_Complex_update_vtable(vt);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
VTABLE* Parrot_Complex_ro_get_vtable(PARROT_INTERP) {
    VTABLE *vt;
    vt = Parrot_default_ro_get_vtable(interp);
    Parrot_Complex_ro_update_vtable(vt);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
PMC* Parrot_Complex_get_mro(PARROT_INTERP, PMC* mro) {
    if (PMC_IS_NULL(mro)) {
        mro = pmc_new(interp, enum_class_ResizableStringArray);
    }

    VTABLE_unshift_string(interp, mro,
        string_make(interp, "Complex", 7, NULL, 0));
    return mro;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
Hash* Parrot_Complex_get_isa(PARROT_INTERP, Hash* isa) {
    if (isa == NULL) {
        isa = parrot_new_hash(interp);
    }

    parrot_hash_put(interp, isa, (void *)(CONST_STRING_GEN(interp, "Complex")), PMCNULL);
    return isa;
}

void
Parrot_Complex_class_init(PARROT_INTERP, int entry, int pass)
{
    static const char attr_defs [] =
        "Fre Fim ";
        STRING * const mfl_0 = CONST_STRING_GEN(interp, "add");
        STRING * const mfl_1 = CONST_STRING_GEN(interp, "PJPPP");
        STRING * const mfl_2 = CONST_STRING_GEN(interp, "Complex,Complex,PMC");
        STRING * const mfl_3 = CONST_STRING_GEN(interp, "Complex");
        STRING * const mfl_4 = CONST_STRING_GEN(interp, "Complex,DEFAULT,PMC");
        STRING * const mfl_5 = CONST_STRING_GEN(interp, "i_add");
        STRING * const mfl_6 = CONST_STRING_GEN(interp, "vJPP");
        STRING * const mfl_7 = CONST_STRING_GEN(interp, "Complex,Complex");
        STRING * const mfl_8 = CONST_STRING_GEN(interp, "Complex,DEFAULT");
        STRING * const mfl_9 = CONST_STRING_GEN(interp, "subtract");
        STRING * const mfl_10 = CONST_STRING_GEN(interp, "i_subtract");
        STRING * const mfl_11 = CONST_STRING_GEN(interp, "multiply");
        STRING * const mfl_12 = CONST_STRING_GEN(interp, "i_multiply");
        STRING * const mfl_13 = CONST_STRING_GEN(interp, "divide");
        STRING * const mfl_14 = CONST_STRING_GEN(interp, "i_divide");
        STRING * const mfl_15 = CONST_STRING_GEN(interp, "is_equal");
        STRING * const mfl_16 = CONST_STRING_GEN(interp, "IJPP");
        STRING * const mfl_17 = CONST_STRING_GEN(interp, "pow");

    const  multi_func_list _temp_multi_func_list[] = {
                { mfl_0,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_add_Complex_PMC }
,
        { mfl_0,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_add_DEFAULT_PMC }
,
        { mfl_5,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_add_Complex }
,
        { mfl_5,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_add_DEFAULT }
,
        { mfl_9,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_subtract_Complex_PMC }
,
        { mfl_9,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_subtract_DEFAULT_PMC }
,
        { mfl_10,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_subtract_Complex }
,
        { mfl_10,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_subtract_DEFAULT }
,
        { mfl_11,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_multiply_Complex_PMC }
,
        { mfl_11,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_multiply_DEFAULT_PMC }
,
        { mfl_12,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_multiply_Complex }
,
        { mfl_12,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_multiply_DEFAULT }
,
        { mfl_13,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_divide_Complex_PMC }
,
        { mfl_13,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_divide_DEFAULT_PMC }
,
        { mfl_14,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_divide_Complex }
,
        { mfl_14,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_divide_DEFAULT }
,
        { mfl_15,
          mfl_16,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_is_equal_Complex }
,
        { mfl_15,
          mfl_16,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_is_equal_DEFAULT }
,
        { mfl_17,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_pow_Complex_PMC }
,
        { mfl_17,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_pow_DEFAULT_PMC }

    };
    if (pass == 0) {
        VTABLE * const vt  = Parrot_Complex_get_vtable(interp);
        vt->base_type      = enum_class_Complex;
        vt->flags          = 0|VTABLE_PMC_NEEDS_EXT|VTABLE_HAS_READONLY_FLAG;
        vt->attribute_defs = attr_defs;
        interp->vtables[entry] = vt;

        vt->whoami       = CONST_STRING_GEN(interp, "Complex");
        vt->provides_str = CONST_STRING_GEN(interp, "scalar");
        vt->isa_hash     = NULL;
        {
            VTABLE                   *vt_ro;
            vt_ro                 = Parrot_Complex_ro_get_vtable(interp);
            vt_ro->base_type      = enum_class_Complex;
            vt_ro->flags          = 0|VTABLE_PMC_NEEDS_EXT|VTABLE_IS_READONLY_FLAG;
            vt_ro->attribute_defs = attr_defs;

            vt_ro->base_type           = entry;
            vt_ro->whoami              = vt->whoami;
            vt_ro->provides_str        = vt->provides_str;
            vt->ro_variant_vtable      = vt_ro;
            vt_ro->ro_variant_vtable = vt;
            vt_ro->isa_hash            = vt->isa_hash;
        }

    }
    else { /* pass */
        {
            VTABLE * const vt  = interp->vtables[entry];

            vt->mro = Parrot_Complex_get_mro(interp, PMCNULL);

            if (vt->ro_variant_vtable)
                vt->ro_variant_vtable->mro = vt->mro;
        }

        /* set up MRO and _namespace */
        Parrot_create_mro(interp, entry);
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_ln), CONST_STRING_GEN(interp, "ln"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_exp), CONST_STRING_GEN(interp, "exp"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sin), CONST_STRING_GEN(interp, "sin"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cos), CONST_STRING_GEN(interp, "cos"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_tan), CONST_STRING_GEN(interp, "tan"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cot), CONST_STRING_GEN(interp, "cot"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sec), CONST_STRING_GEN(interp, "sec"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_csc), CONST_STRING_GEN(interp, "csc"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asin), CONST_STRING_GEN(interp, "asin"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acos), CONST_STRING_GEN(interp, "acos"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_atan), CONST_STRING_GEN(interp, "atan"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acot), CONST_STRING_GEN(interp, "acot"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acsc), CONST_STRING_GEN(interp, "acsc"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asec), CONST_STRING_GEN(interp, "asec"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sinh), CONST_STRING_GEN(interp, "sinh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cosh), CONST_STRING_GEN(interp, "cosh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_tanh), CONST_STRING_GEN(interp, "tanh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_coth), CONST_STRING_GEN(interp, "coth"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_csch), CONST_STRING_GEN(interp, "csch"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sech), CONST_STRING_GEN(interp, "sech"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asinh), CONST_STRING_GEN(interp, "asinh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acosh), CONST_STRING_GEN(interp, "acosh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_atanh), CONST_STRING_GEN(interp, "atanh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acoth), CONST_STRING_GEN(interp, "acoth"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acsch), CONST_STRING_GEN(interp, "acsch"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asech), CONST_STRING_GEN(interp, "asech"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sqrt), CONST_STRING_GEN(interp, "sqrt"));
        {
#define N_MULTI_LIST (sizeof(_temp_multi_func_list)/sizeof(_temp_multi_func_list[0]))
            Parrot_mmd_add_multi_list_from_c_args(interp,
                _temp_multi_func_list, N_MULTI_LIST);
        }
    } /* pass */
} /* Parrot_Complex_class_init */
#line 1914 "./src/pmc/complex.pmc"


/*

=back

=cut

*/

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
